import bls from "@chainsafe/bls/herumi";
import { deriveKeyFromEntropy } from "@chainsafe/bls-keygen";
import { generateKeyPair, generateKeyPairFromSeed } from "@libp2p/crypto/keys";
import { toString as uint8ArrayToString } from "uint8arrays";
import { fromString as uint8ArrayFromString } from "uint8arrays/from-string";
export class DRPCredentialStore {
    _config;
    _ed25519PrivateKey;
    _blsPrivateKey;
    constructor(config) {
        this._config = config;
    }
    async start() {
        if (this._config?.private_key_seed) {
            const tmp = this._config.private_key_seed.padEnd(32, "0");
            const seed = uint8ArrayFromString(tmp);
            this._ed25519PrivateKey = await generateKeyPairFromSeed("Ed25519", seed);
            this._blsPrivateKey = bls.SecretKey.fromBytes(deriveKeyFromEntropy(seed));
        }
        else {
            this._ed25519PrivateKey = await generateKeyPair("Ed25519");
            this._blsPrivateKey = bls.SecretKey.fromKeygen();
        }
    }
    getPublicCredential() {
        if (!this._ed25519PrivateKey || !this._blsPrivateKey) {
            throw new Error("Private key not found");
        }
        return {
            ed25519PublicKey: uint8ArrayToString(this._ed25519PrivateKey?.publicKey.raw, "base64"),
            blsPublicKey: uint8ArrayToString(this._blsPrivateKey?.toPublicKey().toBytes(), "base64"),
        };
    }
    async signWithEd25519(data) {
        if (!this._ed25519PrivateKey) {
            throw new Error("Private key not found");
        }
        const signature = await this._ed25519PrivateKey.sign(uint8ArrayFromString(data));
        return new Uint8Array(signature);
    }
    signWithBls(data) {
        if (!this._blsPrivateKey) {
            throw new Error("Private key not found");
        }
        return this._blsPrivateKey.sign(uint8ArrayFromString(data)).toBytes();
    }
}
